@@ -29,6 +29,7 @@
 #include <stdarg.h>
 #include <assert.h>
 #include <errno.h>
+#include <locale.h>
 
 // project includes
 #include "endianness.h"
@@ -46,9 +47,13 @@
 
 // program information
 #define INFO_PROGRAM "FATSort Utility"
-#define INFO_VERSION "0.9.8.3"
+#define INFO_VERSION "0.9.12"
 #define INFO_AUTHOR "Boris Leidner <fatsort(at)formenos.de>"
 #define INFO_HEADER INFO_PROGRAM " " INFO_VERSION " by " INFO_AUTHOR
+#define INFO_LICENSE	"License GPLv2: GNU GPL version 2 (see LICENSE.txt)\n" \
+			"This is free software: you are free to change and redistribute it.\n" \
+			"There is NO WARRANTY, to the extent permitted by law.\n"
+
 #define INFO_USAGE "\n"	"Usage: fatsort [options] device\n" \
 			"\n" \
 			"Options:\n" \
@@ -57,10 +62,13 @@
 			"\t-h\t Print some help\n" \
 			"\t-i\t Print file system information only\n" \
 			"\t-l\t Print current order of files only\n" \
-			"\t-o flag\t Sort order of files where flag is\n" \
+			"\t-o flag\t Sort order of files where flag is one of\n" \
 			"\t\t\td : directories first (default)\n" \
 			"\t\t\tf : files first\n" \
 			"\t\t\ta : files and directories are not differentiated\n" \
+			"\t-n\t Natural order sorting\n" \
+			"\t-d dir\t Sort directory dir only\n" \
+			"\t-D dir\t Sort directory dir and all subdirectories\n" \
 			"\t-q\t Be quiet\n" \
 			"\t-r\t Sort in reverse order\n" \
 			"\t-v\t Print version information\n" \
@@ -162,19 +170,26 @@ int main(int argc, char *argv[]) {
 	parse arguments and options and start sorting
 */
 
+	// use locale from environment
+	if (setlocale(LC_ALL, "") == NULL) {
+		myerror("Could not set locale!");
+		return -1;
+	}
+
 	// initialize blocked signals
 	init_signal_handling();
 	char *filename;
 
 	if (parse_options(argc, argv) == -1) {
+		myerror("Faild to parse options!");
 		return -1;
 	}
 
 	if (OPT_HELP) {
-		printf(INFO_HEADER INFO_USAGE);
+		printf(INFO_HEADER "\n\n" INFO_LICENSE INFO_USAGE);
 		return 0;
 	} else if (OPT_VERSION) {
-		printf(INFO_HEADER "\n" );
+		printf(INFO_HEADER "\n\n" INFO_LICENSE);
 		return 0;
 	} else if (optind < argc -1) {
 		myerror("Too many arguments!");
