@@ -26,8 +26,11 @@
 
 #include <sys/types.h>
 #include "platform.h"
+#include "FAT_fs.h"
+
+extern u_int32_t OPT_VERSION, OPT_HELP, OPT_INFO, OPT_QUIET, OPT_IGNORE_CASE, OPT_ORDER, OPT_LIST, OPT_REVERSE, OPT_FORCE, OPT_NATURAL_SORT, OPT_RECURSIVE;
+extern char OPT_DIRECTORY[MAX_PATH_LEN+1];
 
-u_int32_t OPT_VERSION, OPT_HELP, OPT_INFO, OPT_QUIET, OPT_IGNORE_CASE, OPT_ORDER, OPT_LIST, OPT_REVERSE, OPT_FORCE;
 
 // parses command line options
 int32_t parse_options(int argc, char *argv[]);
