@@ -31,6 +31,7 @@
 #include <errno.h>
 #include <fcntl.h>
 #include <sys/param.h>
+#include <iconv.h>
 #include "entrylist.h"
 #include "errors.h"
 #include "options.h"
@@ -137,21 +138,44 @@ void parseLongFilenamePart(struct sLongDirEntry *lde, char *str) {
 	assert(lde != NULL);
 	assert(str != NULL);
 
-	u_int32_t len=0;//strlen(str);
-	str[len]=(char) (*(&lde->LDIR_Ord+1));
-	str[len+1]=(char) (*(&lde->LDIR_Ord+3));
-	str[len+2]=(char) (*(&lde->LDIR_Ord+5));
-	str[len+3]=(char) (*(&lde->LDIR_Ord+7));
-	str[len+4]=(char) (*(&lde->LDIR_Ord+9));
-	str[len+5]=(char) (*(&lde->LDIR_Ord+14));
-	str[len+6]=(char) (*(&lde->LDIR_Ord+16));
-	str[len+7]=(char) (*(&lde->LDIR_Ord+18));
-	str[len+8]=(char) (*(&lde->LDIR_Ord+20));
-	str[len+9]=(char) (*(&lde->LDIR_Ord+22));
-	str[len+10]=(char) (*(&lde->LDIR_Ord+24));
-	str[len+11]=(char) (*(&lde->LDIR_Ord+28));
-	str[len+12]=(char) (*(&lde->LDIR_Ord+30));
-	str[len+13]=0;
+	iconv_t cd;
+	size_t incount;
+	size_t outcount;
+	char *outptr = &(str[0]);
+	char utf16str[28];
+	char *inptr=&(utf16str[0]);
+	size_t ret;
+
+	str[0]='\0';
+	
+	memcpy(utf16str, (&lde->LDIR_Ord+1), 10);
+	memcpy(utf16str+10, (&lde->LDIR_Ord+14), 12);
+	memcpy(utf16str+22, (&lde->LDIR_Ord+28), 4);
+	memset(utf16str+26, 0, 2);
+
+	incount=26;
+	outcount=MAX_PATH_LEN;
+
+        cd = iconv_open("UTF-8", "UTF-16");
+
+        if (cd == (iconv_t)-1) {
+                myerror("iconv_open failed!");
+		return;
+        }
+
+	while (incount != 0) {
+                if ((ret=iconv(cd, &inptr, &incount, &outptr, &outcount)) == -1) {
+                        myerror("iconv failed! %d", ret);
+			return;
+                }
+        }
+	outptr[0]='\0';
+
+//	we get a segfault lateron when this function is called
+//	no idea yet
+// 	iconv_close(cd);
+
+	return;
 }
 
 void parseShortFilename(struct sShortDirEntry *sde, char *str) {
@@ -206,7 +230,8 @@ int32_t parseClusterChain(FILE *fd, struct sBootSector *bs, struct sClusterChain
 				myerror("Failed to parse directory entry!");
 				return -1;
 			} else if (ret == 0) {
-				break;
+				//break;
+				return 0;
 			} else if (ret == 2) {
 				parseLongFilenamePart(&de.LongDirEntry, tmp);
 				
@@ -627,14 +652,25 @@ int32_t sortClusterChain(FILE *fd, struct sBootSector *bs, u_int32_t cluster, ch
 	}
 */
 	if (!OPT_LIST) {
+		// DEBUG
+// 		printf("%s %s\n", OPT_DIRECTORY, path);
 		if (parseClusterChain(fd, bs, ClusterChain, list) == -1) {
 			myerror("Failed to parse cluster chain!");
 			return -1;
 		}
-		infomsg("Sorting directory %s\n", path);
-		if (writeClusterChain(fd, bs, list, ClusterChain) == -1) {
-			myerror("Failed to write cluster chain!");
-			return -1;
+
+		if ((strlen(OPT_DIRECTORY) == 0) ||
+		    (strncmp(OPT_DIRECTORY, path, strlen(OPT_DIRECTORY)) == 0)) {
+			infomsg("Sorting directory %s\n", path);
+			if (writeClusterChain(fd, bs, list, ClusterChain) == -1) {
+				myerror("Failed to write cluster chain!");
+				return -1;
+			}
+			if ((strlen(OPT_DIRECTORY) > 0) &&
+			    !OPT_RECURSIVE) {
+				freeClusterChain(ClusterChain);
+				return 0;
+			}
 		}
 	} else {
 		printf("%s\n", path);
@@ -732,14 +768,20 @@ int32_t sort_FAT16_rootdir(FILE *fd, struct sBootSector *bs) {
 	}
 
 	if (!OPT_LIST) {
-		infomsg("Sorting directory /\n");
-
-		fs_seek(fd, BSOffset, SEEK_SET);
-
-		// write the sorted entries back to the fs
-		if (writeList(fd, list) == -1) {
-			myerror("Failed to write FAT entries!");
-			return -1;
+		if ((strlen(OPT_DIRECTORY) == 0) ||
+		    (strncmp(OPT_DIRECTORY, "/", strlen(OPT_DIRECTORY)) == 0)) {
+			infomsg("Sorting directory /\n");
+			fs_seek(fd, BSOffset, SEEK_SET);
+			// write the sorted entries back to the fs
+			if (writeList(fd, list) == -1) {
+			      myerror("Failed to write FAT entries!");
+			      return -1;
+			}
+			if ((strlen(OPT_DIRECTORY) > 0) &&
+			    !OPT_RECURSIVE) {
+				freeDirEntryList(list);
+				return 0;
+			}
 		}
 	} else {
 		printf("\n");
